@@ -7,9 +7,9 @@
  *
  * Purpose: Set/Get the spa encryption type.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@
 #include "fko.h"
 #include "cipher_funcs.h"
 #include "base64.h"
+#include "digest.h"
 
 #if HAVE_LIBGPGME
   #include "gpgme_funcs.h"
@@ -43,120 +44,208 @@
 /* Prep and encrypt using Rijndael
 */
 static int
-_rijndael_encrypt(fko_ctx_t ctx, const char *enc_key)
+_rijndael_encrypt(fko_ctx_t ctx, const char *enc_key, const int enc_key_len)
 {
-    char           *plain;
-    char           *b64cipher;
-    unsigned char  *cipher;
+    char           *plaintext;
+    char           *b64ciphertext;
+    unsigned char  *ciphertext;
     int             cipher_len;
+    int             pt_len;
+    int             zero_free_rv = FKO_SUCCESS;
+
+    if(enc_key_len > RIJNDAEL_MAX_KEYSIZE)
+        return(FKO_ERROR_INVALID_KEY_LEN);
+
+    if (! is_valid_encoded_msg_len(ctx->encoded_msg_len))
+        return(FKO_ERROR_INVALID_DATA);
+
+    switch(ctx->digest_len)
+    {
+        case MD5_B64_LEN:
+            break;
+        case SHA1_B64_LEN:
+            break;
+        case SHA256_B64_LEN:
+            break;
+        case SHA384_B64_LEN:
+            break;
+        case SHA512_B64_LEN:
+            break;
+        default:
+            return(FKO_ERROR_INVALID_DATA);
+    }
+
+    pt_len = ctx->encoded_msg_len + ctx->digest_len + RIJNDAEL_BLOCKSIZE + 2;
 
     /* Make a bucket big enough to hold the enc msg + digest (plaintext)
      * and populate it appropriately.
     */
-    plain = malloc(strlen(ctx->encoded_msg) + strlen(ctx->digest) + 4);
-    if(plain == NULL)
+    plaintext = calloc(1, pt_len);
+
+    if(plaintext == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    snprintf(plain, strlen(ctx->encoded_msg) + strlen(ctx->digest) + 4,
-            "%s:%s", ctx->encoded_msg, ctx->digest);
+    pt_len = snprintf(plaintext, pt_len, "%s:%s", ctx->encoded_msg, ctx->digest);
+
+    if(! is_valid_pt_msg_len(pt_len))
+    {
+        if(zero_free(plaintext, pt_len) == FKO_SUCCESS)
+            return(FKO_ERROR_INVALID_DATA);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+    }
 
     /* Make a bucket for the encrypted version and populate it.
     */
-    cipher = malloc(strlen(plain) + 32); /* Plus padding for salt and Block */
-    if(cipher == NULL)
-        return(FKO_ERROR_MEMORY_ALLOCATION);
+    ciphertext = calloc(1, pt_len + 32); /* Plus padding for salt and Block */
+    if(ciphertext == NULL)
+    {
+        if(zero_free(plaintext, pt_len) == FKO_SUCCESS)
+            return(FKO_ERROR_MEMORY_ALLOCATION);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+    }
 
     cipher_len = rij_encrypt(
-        (unsigned char*)plain, strlen(plain), (char*)enc_key, cipher
+        (unsigned char*)plaintext, pt_len,
+        (char*)enc_key, enc_key_len,
+        ciphertext, ctx->encryption_mode
     );
 
     /* Now make a bucket for the base64-encoded version and populate it.
     */
-    b64cipher = malloc(((cipher_len / 3) * 4) + 8);
-    if(b64cipher == NULL)
-        return(FKO_ERROR_MEMORY_ALLOCATION);
+    b64ciphertext = malloc(((cipher_len / 3) * 4) + 8);
+    if(b64ciphertext == NULL)
+    {
+        if(zero_free((char *) ciphertext, pt_len+32) == FKO_SUCCESS
+                && zero_free(plaintext, pt_len) == FKO_SUCCESS)
+            return(FKO_ERROR_MEMORY_ALLOCATION);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+    }
 
-    b64_encode(cipher, b64cipher, cipher_len);
-    strip_b64_eq(b64cipher);
+    b64_encode(ciphertext, b64ciphertext, cipher_len);
+    strip_b64_eq(b64ciphertext);
 
-    ctx->encrypted_msg = strdup(b64cipher);
+    if(ctx->encrypted_msg != NULL)
+        zero_free_rv = zero_free(ctx->encrypted_msg,
+                strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE));
+
+    ctx->encrypted_msg     = strdup(b64ciphertext);
+    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
 
     /* Clean-up
     */
-    free(plain);
-    free(cipher);
-    free(b64cipher);
+    if(zero_free(plaintext, pt_len) != FKO_SUCCESS)
+        zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+    if(zero_free((char *) ciphertext, pt_len+32) != FKO_SUCCESS)
+        zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+    if(zero_free(b64ciphertext, strnlen(b64ciphertext,
+                    MAX_SPA_ENCODED_MSG_SIZE)) != FKO_SUCCESS)
+        zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
 
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    return(FKO_SUCCESS);
+    if(! is_valid_encoded_msg_len(ctx->encrypted_msg_len))
+        return(FKO_ERROR_INVALID_DATA);
+
+    return(zero_free_rv);
 }
 
 /* Decode, decrypt, and parse SPA data into the context.
 */
 static int
-_rijndael_decrypt(fko_ctx_t ctx, const char *dec_key)
+_rijndael_decrypt(fko_ctx_t ctx,
+    const char *dec_key, const int key_len, int encryption_mode)
 {
-    char           *tbuf;
     unsigned char  *ndx;
     unsigned char  *cipher;
-    int             cipher_len, pt_len, i, err = 0;
+    int             cipher_len, pt_len, i, err = 0, res = FKO_SUCCESS;
+    int             zero_free_rv = FKO_SUCCESS;
 
-    int             b64_len = strlen(ctx->encrypted_msg);
+    if(key_len > RIJNDAEL_MAX_KEYSIZE)
+        return(FKO_ERROR_INVALID_KEY_LEN);
 
     /* Now see if we need to add the "Salted__" string to the front of the
      * encrypted data.
     */
-    if(strncmp(ctx->encrypted_msg, B64_RIJNDAEL_SALT, strlen(B64_RIJNDAEL_SALT)))
+    if(! ctx->added_salted_str)
     {
-        /* We need to realloc space for the salt.
-        */
-        tbuf = realloc(ctx->encrypted_msg, b64_len + 12);
-        if(tbuf == NULL)
-            return(FKO_ERROR_MEMORY_ALLOCATION);
-
-        memmove(tbuf+strlen(B64_RIJNDAEL_SALT), tbuf, b64_len);
-
-        ctx->encrypted_msg = memcpy(tbuf, B64_RIJNDAEL_SALT, strlen(B64_RIJNDAEL_SALT));
-
-        /* Adjust b64_len for added SALT value and Make sure we are still
-         * a properly NULL-terminated string (Ubuntu was one system for
-         * which this was an issue).
-        */
-        b64_len += strlen(B64_RIJNDAEL_SALT);
-        tbuf[b64_len] = '\0';
+        res = add_salted_str(ctx);
+        if(res != FKO_SUCCESS)
+            return res;
     }
 
     /* Create a bucket for the (base64) decoded encrypted data and get the
      * raw cipher data.
     */
-    cipher = malloc(strlen(ctx->encrypted_msg));
+    cipher = malloc(ctx->encrypted_msg_len);
     if(cipher == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
     if((cipher_len = b64_decode(ctx->encrypted_msg, cipher)) < 0)
-        return(FKO_ERROR_INVALID_DATA);
+    {
+        if(zero_free((char *)cipher, ctx->encrypted_msg_len) == FKO_SUCCESS)
+            return(FKO_ERROR_INVALID_DATA);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+    }
+
+    /* Since we're using AES, make sure the incoming data is a multiple of
+     * the blocksize
+    */
+    if((cipher_len % RIJNDAEL_BLOCKSIZE) != 0)
+    {
+        if(zero_free((char *)cipher, ctx->encrypted_msg_len) == FKO_SUCCESS)
+            return(FKO_ERROR_INVALID_DATA);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+    }
+
+    if(ctx->encoded_msg != NULL)
+        zero_free_rv = zero_free(ctx->encoded_msg,
+                strnlen(ctx->encoded_msg, MAX_SPA_ENCODED_MSG_SIZE));
 
     /* Create a bucket for the plaintext data and decrypt the message
      * data into it.
     */
     ctx->encoded_msg = malloc(cipher_len);
     if(ctx->encoded_msg == NULL)
-        return(FKO_ERROR_MEMORY_ALLOCATION);
+    {
+        if(zero_free((char *)cipher, ctx->encrypted_msg_len) == FKO_SUCCESS)
+            return(FKO_ERROR_MEMORY_ALLOCATION);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+    }
 
-    pt_len = rij_decrypt(cipher, cipher_len, dec_key, (unsigned char*)ctx->encoded_msg);
+    pt_len = rij_decrypt(cipher, cipher_len, dec_key, key_len,
+                (unsigned char*)ctx->encoded_msg, encryption_mode);
 
     /* Done with cipher...
     */
-    free(cipher);
+    if(zero_free((char *)cipher, ctx->encrypted_msg_len) != FKO_SUCCESS)
+        zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
 
     /* The length of the decrypted data should be within 32 bytes of the
      * length of the encrypted version.
     */
-    if(pt_len < (cipher_len - 32))
+    if(pt_len < (cipher_len - 32) || pt_len <= 0)
         return(FKO_ERROR_DECRYPTION_SIZE);
 
+    if(ctx->encoded_msg == NULL)
+        return(FKO_ERROR_INVALID_DATA);
+
+    if(! is_valid_encoded_msg_len(pt_len))
+        return(FKO_ERROR_INVALID_DATA);
+
+    if(zero_free_rv != FKO_SUCCESS)
+        return(zero_free_rv);
+
+    ctx->encoded_msg_len = pt_len;
+
     /* At this point we can check the data to see if we have a good
      * decryption by ensuring the first field (16-digit random decimal
      * value) is valid and is followed by a colon.  Additional checks
@@ -185,63 +274,131 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
 {
     int             res;
     char           *plain;
+    int             pt_len, zero_free_rv = FKO_SUCCESS;
     char           *b64cipher;
     unsigned char  *cipher = NULL;
     size_t          cipher_len;
+    char           *empty_key = "";
+
+    if (! is_valid_encoded_msg_len(ctx->encoded_msg_len))
+        return(FKO_ERROR_INVALID_DATA);
+
+    switch(ctx->digest_len)
+    {
+        case MD5_B64_LEN:
+            break;
+        case SHA1_B64_LEN:
+            break;
+        case SHA256_B64_LEN:
+            break;
+        case SHA384_B64_LEN:
+            break;
+        case SHA512_B64_LEN:
+            break;
+        default:
+            return(FKO_ERROR_INVALID_DATA);
+    }
 
     /* First make sure we have a recipient key set.
     */
     if(ctx->gpg_recipient == NULL)
         return(FKO_ERROR_MISSING_GPG_KEY_DATA);
 
+    pt_len = ctx->encoded_msg_len + ctx->digest_len + 2;
+
     /* Make a bucket big enough to hold the enc msg + digest (plaintext)
      * and populate it appropriately.
     */
-    plain = malloc(strlen(ctx->encoded_msg) + strlen(ctx->digest) + 2);
+    plain = malloc(ctx->encoded_msg_len + ctx->digest_len + 2);
     if(plain == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    snprintf(plain, strlen(ctx->encoded_msg) + strlen(ctx->digest) + 2,
-            "%s:%s", ctx->encoded_msg, ctx->digest);
+    pt_len = snprintf(plain, pt_len+1, "%s:%s", ctx->encoded_msg, ctx->digest);
 
-    res = gpgme_encrypt(ctx,
-        (unsigned char*)plain, strlen(plain),
-        enc_key, &cipher, &cipher_len
-    );
+    if(! is_valid_pt_msg_len(pt_len))
+    {
+        if(zero_free(plain, pt_len) == FKO_SUCCESS)
+            return(FKO_ERROR_INVALID_DATA);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+    }
+
+    if (enc_key != NULL)
+    {
+        res = gpgme_encrypt(ctx, (unsigned char*)plain, pt_len,
+            enc_key, &cipher, &cipher_len
+        );
+    }
+    else
+    {
+        res = gpgme_encrypt(ctx, (unsigned char*)plain, pt_len,
+            empty_key, &cipher, &cipher_len
+        );
+    }
 
     /* --DSS XXX: Better parsing of what went wrong would be nice :)
     */
     if(res != FKO_SUCCESS)
     {
-        free(plain);
+        zero_free_rv = zero_free(plain, pt_len);
 
-        if(cipher)
-            free(cipher);
+        if(cipher != NULL)
+            if(zero_free((char *) cipher, cipher_len) != FKO_SUCCESS)
+                zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
 
-        return(res);
+        if(zero_free_rv == FKO_SUCCESS)
+            return(res);
+        else
+            return(zero_free_rv);
     }
 
     /* Now make a bucket for the base64-encoded version and populate it.
     */
     b64cipher = malloc(((cipher_len / 3) * 4) + 8);
     if(b64cipher == NULL)
-        return(FKO_ERROR_MEMORY_ALLOCATION);
+    {
+        if(zero_free(plain, pt_len) != FKO_SUCCESS)
+            zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+        if(cipher != NULL)
+            if(zero_free((char *) cipher, cipher_len) != FKO_SUCCESS)
+                zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+        if(zero_free_rv == FKO_SUCCESS)
+            return(FKO_ERROR_MEMORY_ALLOCATION);
+        else
+            return(zero_free_rv);
+    }
 
     b64_encode(cipher, b64cipher, cipher_len);
     strip_b64_eq(b64cipher);
 
-    ctx->encrypted_msg = strdup(b64cipher);
+    if(ctx->encrypted_msg != NULL)
+        zero_free_rv = zero_free(ctx->encrypted_msg,
+                strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE));
+
+    ctx->encrypted_msg     = strdup(b64cipher);
+    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
 
     /* Clean-up
     */
-    free(plain);
-    free(cipher);
-    free(b64cipher);
+    if(zero_free(plain, pt_len) != FKO_SUCCESS)
+        zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+    if(zero_free((char *) cipher, cipher_len) != FKO_SUCCESS)
+        zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+    if(zero_free(b64cipher, strnlen(b64cipher,
+                    MAX_SPA_ENCODED_MSG_SIZE)) != FKO_SUCCESS)
+        zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
 
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    return(FKO_SUCCESS);
+    if(! is_valid_encoded_msg_len(ctx->encrypted_msg_len))
+        return(FKO_ERROR_INVALID_DATA);
+
+    return(zero_free_rv);
 }
 
 /* Prep and decrypt using gpgme
@@ -249,45 +406,33 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
 static int
 gpg_decrypt(fko_ctx_t ctx, const char *dec_key)
 {
-    char           *tbuf;
     unsigned char  *cipher;
     size_t          cipher_len;
-    int             res;
-
-    int             b64_len = strlen(ctx->encrypted_msg);
+    int             res, pt_len, b64_decode_len;
 
     /* Now see if we need to add the "hQ" string to the front of the
      * base64-encoded-GPG-encrypted data.
     */
-    if(strncmp(ctx->encrypted_msg, B64_GPG_PREFIX, strlen(B64_GPG_PREFIX)))
-    {
-        /* We need to realloc space for the GPG prefix of hQ.
-        */
-        tbuf = realloc(ctx->encrypted_msg, b64_len + 12);
-        if(tbuf == NULL)
-            return(FKO_ERROR_MEMORY_ALLOCATION);
-
-        memmove(tbuf+strlen(B64_GPG_PREFIX), tbuf, b64_len);
-
-        ctx->encrypted_msg = memcpy(tbuf, B64_GPG_PREFIX, strlen(B64_GPG_PREFIX));
-
-        /* Adjust b64_len for added SALT value and Make sure we are still
-         * a properly NULL-terminated string (Ubuntu was one system for
-         * which this was an issue).
-        */
-        b64_len += strlen(B64_GPG_PREFIX);
-        tbuf[b64_len] = '\0';
-    }
+    if(! ctx->added_gpg_prefix)
+        add_gpg_prefix(ctx);
 
     /* Create a bucket for the (base64) decoded encrypted data and get the
      * raw cipher data.
     */
-    cipher = malloc(strlen(ctx->encrypted_msg));
+    cipher = malloc(ctx->encrypted_msg_len);
     if(cipher == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
-    if((cipher_len = b64_decode(ctx->encrypted_msg, cipher)) < 0)
-        return(FKO_ERROR_INVALID_DATA);
+    if((b64_decode_len = b64_decode(ctx->encrypted_msg, cipher)) < 0)
+    {
+        if(zero_free((char *) cipher, ctx->encrypted_msg_len) == FKO_SUCCESS)
+            return(FKO_ERROR_INVALID_DATA);
+        else
+            return(FKO_ERROR_ZERO_OUT_DATA);
+
+    }
+
+    cipher_len = b64_decode_len;
 
     /* Create a bucket for the plaintext data and decrypt the message
      * data into it.
@@ -300,19 +445,28 @@ gpg_decrypt(fko_ctx_t ctx, const char *dec_key)
     */
 
     res = gpgme_decrypt(ctx, cipher, cipher_len,
-        dec_key,  (unsigned char**)&ctx->encoded_msg, &cipher_len
+        dec_key, (unsigned char**)&ctx->encoded_msg, &cipher_len
     );
 
     /* Done with cipher...
     */
-    free(cipher);
-
     if(res != FKO_SUCCESS)
-        return(res);
+    {
+        if(zero_free((char *) cipher, ctx->encrypted_msg_len) != FKO_SUCCESS)
+            return(FKO_ERROR_ZERO_OUT_DATA);
+        else
+            return(res);
+    }
 
-    /* XXX: We could put some kind of sanity check  of the decrypted
-     *      data here
-    */
+    pt_len = strnlen(ctx->encoded_msg, MAX_SPA_ENCODED_MSG_SIZE);
+
+    if(ctx->encoded_msg == NULL)
+        return(FKO_ERROR_INVALID_DATA);
+
+    if(! is_valid_encoded_msg_len(pt_len))
+        return(FKO_ERROR_INVALID_DATA);
+
+    ctx->encoded_msg_len = pt_len;
 
     /* Call fko_decode and return the results.
     */
@@ -356,19 +510,53 @@ fko_get_spa_encryption_type(fko_ctx_t ctx, short *enc_type)
     return(FKO_SUCCESS);
 }
 
+/* Set the SPA encryption mode.
+*/
+int
+fko_set_spa_encryption_mode(fko_ctx_t ctx, const int encrypt_mode)
+{
+    /* Must be initialized
+    */
+    if(!CTX_INITIALIZED(ctx))
+        return(FKO_ERROR_CTX_NOT_INITIALIZED);
+
+    if(encrypt_mode < 0 || encrypt_mode >= FKO_LAST_ENC_MODE)
+        return(FKO_ERROR_INVALID_DATA);
+
+    ctx->encryption_mode = encrypt_mode;
+
+    ctx->state |= FKO_ENCRYPT_MODE_MODIFIED;
+
+    return(FKO_SUCCESS);
+}
+
+/* Return the SPA encryption mode.
+*/
+int
+fko_get_spa_encryption_mode(fko_ctx_t ctx, int *enc_mode)
+{
+    /* Must be initialized
+    */
+    if(!CTX_INITIALIZED(ctx))
+        return(FKO_ERROR_CTX_NOT_INITIALIZED);
+
+    *enc_mode = ctx->encryption_mode;
+
+    return(FKO_SUCCESS);
+}
+
 /* Encrypt the encoded SPA data.
 */
 int
-fko_encrypt_spa_data(fko_ctx_t ctx, const char *enc_key)
+fko_encrypt_spa_data(fko_ctx_t ctx, const char * const enc_key,
+        const int enc_key_len)
 {
     int             res = 0;
 
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
-    {
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
-    }
 
     /* If there is no encoded data or the SPA data has been modified,
      * go ahead and re-encode here.
@@ -383,15 +571,13 @@ fko_encrypt_spa_data(fko_ctx_t ctx, const char *enc_key)
      * check for a somewhat arbitrary minimum length for the encoded
      * data.
     */
-    if(strlen(ctx->encoded_msg) < MIN_SPA_ENCODED_MSG_SIZE)
-    {
+    if (! is_valid_encoded_msg_len(ctx->encoded_msg_len))
         return(FKO_ERROR_MISSING_ENCODED_DATA);
-    }
 
     /* Encrypt according to type and return...
     */
     if(ctx->encryption_type == FKO_ENCRYPTION_RIJNDAEL)
-        res = _rijndael_encrypt(ctx, enc_key);
+        res = _rijndael_encrypt(ctx, enc_key, enc_key_len);
     else if(ctx->encryption_type == FKO_ENCRYPTION_GPG)
 #if HAVE_LIBGPGME
         res = gpg_encrypt(ctx, enc_key);
@@ -407,17 +593,18 @@ fko_encrypt_spa_data(fko_ctx_t ctx, const char *enc_key)
 /* Decode, decrypt, and parse SPA data into the context.
 */
 int
-fko_decrypt_spa_data(fko_ctx_t ctx, const char *dec_key)
+fko_decrypt_spa_data(fko_ctx_t ctx, const char * const dec_key, const int key_len)
 {
     int     enc_type, res;
 
+    if(!CTX_INITIALIZED(ctx))
+        return(FKO_ERROR_CTX_NOT_INITIALIZED);
+
     /* Get the (assumed) type of encryption used. This will also provide
      * some data validation.
     */
     enc_type = fko_encryption_type(ctx->encrypted_msg);
 
-    //strlen(ctx->encrypted_msg) <  MIN_SPA_ENCODED_MSG_SIZE)
-
     if(enc_type == FKO_ENCRYPTION_GPG)
     {
         ctx->encryption_type = FKO_ENCRYPTION_GPG;
@@ -430,7 +617,8 @@ fko_decrypt_spa_data(fko_ctx_t ctx, const char *dec_key)
     else if(enc_type == FKO_ENCRYPTION_RIJNDAEL)
     {
         ctx->encryption_type = FKO_ENCRYPTION_RIJNDAEL;
-        res = _rijndael_decrypt(ctx, dec_key);
+        res = _rijndael_decrypt(ctx,
+            dec_key, key_len, ctx->encryption_mode);
     }
     else
         return(FKO_ERROR_INVALID_DATA);
@@ -441,7 +629,7 @@ fko_decrypt_spa_data(fko_ctx_t ctx, const char *dec_key)
 /* Return the assumed encryption type based on the raw encrypted data.
 */
 int
-fko_encryption_type(const char *enc_data)
+fko_encryption_type(const char * const enc_data)
 {
     int enc_data_len;
 
@@ -450,13 +638,10 @@ fko_encryption_type(const char *enc_data)
     if(enc_data == NULL)
         return(FKO_ENCRYPTION_INVALID_DATA);
 
-    /* Determine type of encryption used.  For now, we are using the
-     * size of the message.
-     *
-     * XXX: We will want to come up with a more reliable method of
-     *      identifying the encryption type.
-    */
-    enc_data_len = strlen(enc_data);
+    enc_data_len = strnlen(enc_data, MAX_SPA_ENCODED_MSG_SIZE);
+
+    if(! is_valid_encoded_msg_len(enc_data_len))
+        return(FKO_ENCRYPTION_UNKNOWN);
 
     if(enc_data_len >= MIN_GNUPG_MSG_SIZE)
         return(FKO_ENCRYPTION_GPG);
@@ -472,7 +657,7 @@ fko_encryption_type(const char *enc_data)
 /* Set the GPG recipient key name.
 */
 int
-fko_set_gpg_recipient(fko_ctx_t ctx, const char *recip)
+fko_set_gpg_recipient(fko_ctx_t ctx, const char * const recip)
 {
 #if HAVE_LIBGPGME
     int             res;
@@ -486,6 +671,9 @@ fko_set_gpg_recipient(fko_ctx_t ctx, const char *recip)
     if(ctx->encryption_type != FKO_ENCRYPTION_GPG)
         return(FKO_ERROR_WRONG_ENCRYPTION_TYPE);
 
+    if(ctx->gpg_recipient != NULL)
+        free(ctx->gpg_recipient);
+
     ctx->gpg_recipient = strdup(recip);
     if(ctx->gpg_recipient == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
@@ -513,7 +701,7 @@ fko_set_gpg_recipient(fko_ctx_t ctx, const char *recip)
 /* Set the GPG home dir.
 */
 int
-fko_set_gpg_exe(fko_ctx_t ctx, const char *gpg_exe)
+fko_set_gpg_exe(fko_ctx_t ctx, const char * const gpg_exe)
 {
 #if HAVE_LIBGPGME
     struct stat     st;
@@ -532,6 +720,9 @@ fko_set_gpg_exe(fko_ctx_t ctx, const char *gpg_exe)
     if(!S_ISREG(st.st_mode) && !S_ISLNK(st.st_mode))
         return(FKO_ERROR_GPGME_BAD_GPG_EXE);
 
+    if(ctx->gpg_exe != NULL)
+        free(ctx->gpg_exe);
+
     ctx->gpg_exe = strdup(gpg_exe);
     if(ctx->gpg_exe == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
@@ -583,7 +774,7 @@ fko_get_gpg_recipient(fko_ctx_t ctx, char **recipient)
 /* Set the GPG signer key name.
 */
 int
-fko_set_gpg_signer(fko_ctx_t ctx, const char *signer)
+fko_set_gpg_signer(fko_ctx_t ctx, const char * const signer)
 {
 #if HAVE_LIBGPGME
     int             res;
@@ -597,6 +788,9 @@ fko_set_gpg_signer(fko_ctx_t ctx, const char *signer)
     if(ctx->encryption_type != FKO_ENCRYPTION_GPG)
         return(FKO_ERROR_WRONG_ENCRYPTION_TYPE);
 
+    if(ctx->gpg_signer != NULL)
+        free(ctx->gpg_signer);
+
     ctx->gpg_signer = strdup(signer);
     if(ctx->gpg_signer == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
@@ -643,7 +837,7 @@ fko_get_gpg_signer(fko_ctx_t ctx, char **signer)
 /* Set the GPG home dir.
 */
 int
-fko_set_gpg_home_dir(fko_ctx_t ctx, const char *gpg_home_dir)
+fko_set_gpg_home_dir(fko_ctx_t ctx, const char * const gpg_home_dir)
 {
 #if HAVE_LIBGPGME
     struct stat     st;
@@ -661,6 +855,9 @@ fko_set_gpg_home_dir(fko_ctx_t ctx, const char *gpg_home_dir)
     if(!S_ISDIR(st.st_mode))
         return(FKO_ERROR_GPGME_BAD_HOME_DIR);
 
+    if(ctx->gpg_home_dir != NULL)
+        free(ctx->gpg_home_dir);
+
     ctx->gpg_home_dir = strdup(gpg_home_dir);
     if(ctx->gpg_home_dir == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
@@ -708,7 +905,7 @@ fko_set_gpg_signature_verify(fko_ctx_t ctx, const unsigned char val)
 }
 
 int
-fko_get_gpg_signature_verify(fko_ctx_t ctx, unsigned char *val)
+fko_get_gpg_signature_verify(fko_ctx_t ctx, unsigned char * const val)
 {
 #if HAVE_LIBGPGME
     /* Must be initialized
@@ -742,7 +939,7 @@ fko_set_gpg_ignore_verify_error(fko_ctx_t ctx, const unsigned char val)
 }
 
 int
-fko_get_gpg_ignore_verify_error(fko_ctx_t ctx, unsigned char *val)
+fko_get_gpg_ignore_verify_error(fko_ctx_t ctx, unsigned char * const val)
 {
 #if HAVE_LIBGPGME
     /* Must be initialized
@@ -888,7 +1085,8 @@ fko_get_gpg_signature_status(fko_ctx_t ctx, int *sigstat)
 }
 
 int
-fko_gpg_signature_id_match(fko_ctx_t ctx, const char *id, unsigned char *result)
+fko_gpg_signature_id_match(fko_ctx_t ctx, const char * const id,
+        unsigned char * const result)
 {
 #if HAVE_LIBGPGME
     char *curr_id;
@@ -924,7 +1122,8 @@ fko_gpg_signature_id_match(fko_ctx_t ctx, const char *id, unsigned char *result)
 }
 
 int
-fko_gpg_signature_fpr_match(fko_ctx_t ctx, const char *id, unsigned char *result)
+fko_gpg_signature_fpr_match(fko_ctx_t ctx, const char * const id,
+        unsigned char * const result)
 {
 #if HAVE_LIBGPGME
     /* Must be initialized
