@@ -7,9 +7,9 @@
  *
  * Purpose: Error handling functions for libfko
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -60,6 +60,9 @@ fko_errstr(const int err_code)
         case FKO_ERROR_DATA_TOO_LARGE:
             return("Value or Size of the data exceeded the max allowed");
 
+        case FKO_ERROR_INVALID_KEY_LEN:
+            return("Invalid key length");
+
         case FKO_ERROR_USERNAME_UNKNOWN:
             return("Unable to determine username");
 
@@ -99,9 +102,18 @@ fko_errstr(const int err_code)
         case FKO_ERROR_DIGEST_VERIFICATION_FAILED:
             return("The computed digest did not match the digest in the spa data");
 
+        case FKO_ERROR_INVALID_HMAC_KEY_LEN:
+            return("Invalid HMAC key length");
+
+        case FKO_ERROR_UNSUPPORTED_HMAC_MODE:
+            return("Unsupported HMAC mode (default: SHA256)");
+
         case FKO_ERROR_UNSUPPORTED_FEATURE:
             return("Unsupported or unimplemented feature or function");
 
+        case FKO_ERROR_ZERO_OUT_DATA:
+            return("Could not zero out sensitive data");
+
         case FKO_ERROR_UNKNOWN:
             return("Unknown/Unclassified error");
 
@@ -198,6 +210,12 @@ const char*
 fko_gpg_errstr(fko_ctx_t ctx)
 {
 #if HAVE_LIBGPGME
+
+    /* Must be initialized
+     */
+    if(!CTX_INITIALIZED(ctx))
+        return("");
+
     if(ctx->gpg_err)
         return(gpgme_strerror(ctx->gpg_err));
 #endif /* HAVE_LIBGPGME */
