@@ -7,9 +7,9 @@
  *
  * Purpose: General utility functions for libfko
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -31,13 +31,15 @@
 #include "fko_common.h"
 #include "fko.h"
 #include "cipher_funcs.h"
+#include "base64.h"
+#include "digest.h"
 
 /* Initialize an fko context.
 */
 int
 fko_new(fko_ctx_t *r_ctx)
 {
-    fko_ctx_t   ctx;
+    fko_ctx_t   ctx = NULL;
     int         res;
     char       *ver;
 
@@ -75,6 +77,7 @@ fko_new(fko_ctx_t *r_ctx)
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
+        ctx = NULL;
         return res;
     }
 
@@ -86,6 +89,7 @@ fko_new(fko_ctx_t *r_ctx)
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
+        ctx = NULL;
         return res;
     }
 
@@ -97,6 +101,7 @@ fko_new(fko_ctx_t *r_ctx)
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
+        ctx = NULL;
         return res;
     }
 
@@ -108,6 +113,7 @@ fko_new(fko_ctx_t *r_ctx)
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
+        ctx = NULL;
         return res;
     }
 
@@ -119,6 +125,7 @@ fko_new(fko_ctx_t *r_ctx)
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
+        ctx = NULL;
         return res;
     }
 
@@ -130,6 +137,19 @@ fko_new(fko_ctx_t *r_ctx)
     if(res != FKO_SUCCESS)
     {
         fko_destroy(ctx);
+        ctx = NULL;
+        return res;
+    }
+
+    /* Default is Rijndael in CBC mode
+    */
+    ctx->initval = FKO_CTX_INITIALIZED;
+    res = fko_set_spa_encryption_mode(ctx, FKO_DEFAULT_ENC_MODE);
+    ctx->initval = 0;
+    if(res != FKO_SUCCESS)
+    {
+        fko_destroy(ctx);
+        ctx = NULL;
         return res;
     }
 
@@ -156,39 +176,96 @@ fko_new(fko_ctx_t *r_ctx)
  * and parsing the provided data into the context data.
 */
 int
-fko_new_with_data(fko_ctx_t *r_ctx, const char *enc_msg, const char *dec_key)
+fko_new_with_data(fko_ctx_t *r_ctx, const char * const enc_msg,
+    const char * const dec_key, const int dec_key_len,
+    int encryption_mode, const char * const hmac_key,
+    const int hmac_key_len, const int hmac_type)
 {
-    fko_ctx_t   ctx;
+    fko_ctx_t   ctx = NULL;
     int         res = FKO_SUCCESS; /* Are we optimistic or what? */
+    int         enc_msg_len;
+
+    if(enc_msg == NULL)
+        return(FKO_ERROR_INVALID_DATA);
 
     ctx = calloc(1, sizeof *ctx);
     if(ctx == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
+    enc_msg_len = strnlen(enc_msg, MAX_SPA_ENCODED_MSG_SIZE);
+
+    if(! is_valid_encoded_msg_len(enc_msg_len))
+    {
+        free(ctx);
+        return(FKO_ERROR_INVALID_DATA);
+    }
+
+    if(ctx->encrypted_msg != NULL)
+        free(ctx->encrypted_msg);
+
     /* First, add the data to the context.
     */
-    ctx->encrypted_msg = strdup(enc_msg);
+    ctx->encrypted_msg     = strdup(enc_msg);
+    ctx->encrypted_msg_len = enc_msg_len;
+
     if(ctx->encrypted_msg == NULL)
     {
         free(ctx);
         return(FKO_ERROR_MEMORY_ALLOCATION);
     }
 
+    /* Default Encryption Mode (Rijndael in CBC mode)
+    */
+    ctx->initval = FKO_CTX_INITIALIZED;
+    res = fko_set_spa_encryption_mode(ctx, encryption_mode);
+    if(res != FKO_SUCCESS)
+    {
+        fko_destroy(ctx);
+        ctx = NULL;
+        return res;
+    }
+    ctx->initval = 0;
+
+    /* HMAC digest type
+    */
+    ctx->initval = FKO_CTX_INITIALIZED;
+    res = fko_set_spa_hmac_type(ctx, hmac_type);
+    if(res != FKO_SUCCESS)
+    {
+        fko_destroy(ctx);
+        ctx = NULL;
+        return res;
+    }
+    ctx->initval = 0;
+
+    /* Check HMAC if the access stanza had an HMAC key
+    */
+    ctx->initval = FKO_CTX_INITIALIZED;
+    if(hmac_key_len > 0 && hmac_key != NULL)
+        res = fko_verify_hmac(ctx, hmac_key, hmac_key_len);
+    if(res != FKO_SUCCESS)
+    {
+        fko_destroy(ctx);
+        ctx = NULL;
+        return res;
+    }
+    ctx->initval = 0;
+
     /* Consider it initialized here.
     */
     ctx->initval = FKO_CTX_INITIALIZED;
     FKO_SET_CTX_INITIALIZED(ctx);
 
-    /* If a decryption password is provided, go ahead and decrypt and
-     * decode.
+    /* If a decryption key is provided, go ahead and decrypt and decode.
     */
     if(dec_key != NULL)
     {
-        res = fko_decrypt_spa_data(ctx, dec_key);
+        res = fko_decrypt_spa_data(ctx, dec_key, dec_key_len);
 
         if(res != FKO_SUCCESS)
         {
             fko_destroy(ctx);
+            ctx = NULL;
             *r_ctx = NULL; /* Make sure the caller ctx is null just in case */
             return(res);
         }
@@ -208,89 +285,163 @@ fko_new_with_data(fko_ctx_t *r_ctx, const char *enc_msg, const char *dec_key)
 
 /* Destroy a context and free its resources
 */
-void
+int
 fko_destroy(fko_ctx_t ctx)
 {
+    int zero_free_rv = FKO_SUCCESS;
+
 #if HAVE_LIBGPGME
     fko_gpg_sig_t   gsig, tgsig;
 #endif
 
-    if(CTX_INITIALIZED(ctx))
-    {
-        if(ctx->rand_val != NULL)
-            free(ctx->rand_val);
+    if(!CTX_INITIALIZED(ctx))
+        return(zero_free_rv);
 
-        if(ctx->username != NULL)
-            free(ctx->username);
+    if(ctx->rand_val != NULL)
+        free(ctx->rand_val);
 
-        if(ctx->version != NULL)
-            free(ctx->version);
+    if(ctx->username != NULL)
+        free(ctx->username);
 
-        if(ctx->message != NULL)
-            free(ctx->message);
+    if(ctx->version != NULL)
+        free(ctx->version);
 
-        if(ctx->nat_access != NULL)
-            free(ctx->nat_access);
+    if(ctx->message != NULL)
+        free(ctx->message);
 
-        if(ctx->server_auth != NULL)
-            free(ctx->server_auth);
+    if(ctx->nat_access != NULL)
+        free(ctx->nat_access);
 
-        if(ctx->digest != NULL)
-            free(ctx->digest);
+    if(ctx->server_auth != NULL)
+        free(ctx->server_auth);
 
-        if(ctx->raw_digest != NULL)
-            free(ctx->raw_digest);
+    if(ctx->digest != NULL)
+        if(zero_free(ctx->digest, ctx->digest_len) != FKO_SUCCESS)
+            zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
 
-        if(ctx->encoded_msg != NULL)
-            free(ctx->encoded_msg);
+    if(ctx->raw_digest != NULL)
+        if(zero_free(ctx->raw_digest, ctx->raw_digest_len) != FKO_SUCCESS)
+            zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
 
-        if(ctx->encrypted_msg != NULL)
-            free(ctx->encrypted_msg);
+    if(ctx->encoded_msg != NULL)
+        if(zero_free(ctx->encoded_msg, ctx->encoded_msg_len) != FKO_SUCCESS)
+            zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+    if(ctx->encrypted_msg != NULL)
+        if(zero_free(ctx->encrypted_msg, ctx->encrypted_msg_len) != FKO_SUCCESS)
+            zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
+
+    if(ctx->msg_hmac != NULL)
+        if(zero_free(ctx->msg_hmac, ctx->msg_hmac_len) != FKO_SUCCESS)
+            zero_free_rv = FKO_ERROR_ZERO_OUT_DATA;
 
 #if HAVE_LIBGPGME
-        if(ctx->gpg_exe != NULL)
-            free(ctx->gpg_exe);
+    if(ctx->gpg_exe != NULL)
+        free(ctx->gpg_exe);
 
-        if(ctx->gpg_home_dir != NULL)
-            free(ctx->gpg_home_dir);
+    if(ctx->gpg_home_dir != NULL)
+        free(ctx->gpg_home_dir);
 
-        if(ctx->gpg_recipient != NULL)
-            free(ctx->gpg_recipient);
+    if(ctx->gpg_recipient != NULL)
+        free(ctx->gpg_recipient);
 
-        if(ctx->gpg_signer != NULL)
-            free(ctx->gpg_signer);
+    if(ctx->gpg_signer != NULL)
+        free(ctx->gpg_signer);
 
-        if(ctx->recipient_key != NULL)
-        {
-            gpgme_key_unref(ctx->recipient_key);
-        }
+    if(ctx->recipient_key != NULL)
+        gpgme_key_unref(ctx->recipient_key);
 
-        if(ctx->signer_key != NULL)
-        {
-            gpgme_key_unref(ctx->signer_key);
-        }
+    if(ctx->signer_key != NULL)
+        gpgme_key_unref(ctx->signer_key);
 
-        if(ctx->gpg_ctx != NULL)
-            gpgme_release(ctx->gpg_ctx);
+    if(ctx->gpg_ctx != NULL)
+        gpgme_release(ctx->gpg_ctx);
 
-        gsig = ctx->gpg_sigs;
-        while(gsig != NULL)
-        {
-            if(gsig->fpr != NULL)
-                free(gsig->fpr);
+    gsig = ctx->gpg_sigs;
+    while(gsig != NULL)
+    {
+        if(gsig->fpr != NULL)
+            free(gsig->fpr);
 
-            tgsig = gsig;
-            gsig = gsig->next;
+        tgsig = gsig;
+        gsig = gsig->next;
 
-            free(tgsig);
-        }
+        free(tgsig);
+    }
 
 #endif /* HAVE_LIBGPGME */
 
-        bzero(ctx, sizeof(*ctx));
-    }
+    memset(ctx, 0x0, sizeof(*ctx));
 
     free(ctx);
+
+    return(zero_free_rv);
+}
+
+/* Generate Rijndael and HMAC keys from /dev/random and base64
+ * encode them
+*/
+int
+fko_key_gen(char * const key_base64, const int key_len,
+        char * const hmac_key_base64, const int hmac_key_len,
+        const int hmac_type)
+{
+    unsigned char key[RIJNDAEL_MAX_KEYSIZE];
+    unsigned char hmac_key[SHA512_BLOCK_LEN];
+    int klen      = key_len;
+    int hmac_klen = hmac_key_len;
+    int b64_len   = 0;
+
+    if(key_len == FKO_DEFAULT_KEY_LEN)
+        klen = RIJNDAEL_MAX_KEYSIZE;
+
+    if(hmac_key_len == FKO_DEFAULT_KEY_LEN)
+    {
+        if(hmac_type == FKO_DEFAULT_HMAC_MODE
+                || hmac_type == FKO_HMAC_SHA256)
+            hmac_klen = SHA256_BLOCK_LEN;
+        else if(hmac_type == FKO_HMAC_MD5)
+            hmac_klen = MD5_DIGEST_LEN;
+        else if(hmac_type == FKO_HMAC_SHA1)
+            hmac_klen = SHA1_DIGEST_LEN;
+        else if(hmac_type == FKO_HMAC_SHA384)
+            hmac_klen = SHA384_BLOCK_LEN;
+        else if(hmac_type == FKO_HMAC_SHA512)
+            hmac_klen = SHA512_BLOCK_LEN;
+    }
+
+    if((klen < 1) || (klen > RIJNDAEL_MAX_KEYSIZE))
+        return(FKO_ERROR_INVALID_DATA);
+
+    if((hmac_klen < 1) || (hmac_klen > SHA512_BLOCK_LEN))
+        return(FKO_ERROR_INVALID_DATA);
+
+    get_random_data(key, klen);
+    get_random_data(hmac_key, hmac_klen);
+
+    b64_len = b64_encode(key, key_base64, klen);
+    if(b64_len < klen)
+        return(FKO_ERROR_INVALID_DATA);
+
+    b64_len = b64_encode(hmac_key, hmac_key_base64, hmac_klen);
+    if(b64_len < hmac_klen)
+        return(FKO_ERROR_INVALID_DATA);
+
+    return(FKO_SUCCESS);
+}
+
+/* Provide an FKO wrapper around base64 encode/decode functions
+*/
+int
+fko_base64_encode(unsigned char * const in, char * const out, int in_len)
+{
+    return b64_encode(in, out, in_len);
+}
+
+int
+fko_base64_decode(const char * const in, unsigned char *out)
+{
+    return b64_decode(in, out);
 }
 
 /* Return the fko version
@@ -313,14 +464,48 @@ fko_get_version(fko_ctx_t ctx, char **version)
  * set.
 */
 int
-fko_spa_data_final(fko_ctx_t ctx, const char *enc_key)
+fko_spa_data_final(fko_ctx_t ctx,
+    const char * const enc_key, const int enc_key_len,
+    const char * const hmac_key, const int hmac_key_len)
 {
+    char   *tbuf;
+    int     res = 0, data_with_hmac_len = 0;
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
-    return(fko_encrypt_spa_data(ctx, enc_key));
+    res = fko_encrypt_spa_data(ctx, enc_key, enc_key_len);
+
+    /* Now calculate hmac if so configured
+    */
+    if (res == FKO_SUCCESS &&
+            ctx->hmac_type != FKO_HMAC_UNKNOWN && hmac_key != NULL)
+    {
+        res = fko_set_spa_hmac(ctx, hmac_key, hmac_key_len);
+
+        if (res == FKO_SUCCESS)
+        {
+            /* Now that we have the hmac, append it to the
+             * encrypted data (which has already been base64-encoded
+             * and the trailing '=' chars stripped off).
+            */
+            data_with_hmac_len
+                = ctx->encrypted_msg_len+1+ctx->msg_hmac_len+1;
+
+            tbuf = realloc(ctx->encrypted_msg, data_with_hmac_len);
+            if (tbuf == NULL)
+                return(FKO_ERROR_MEMORY_ALLOCATION);
+
+            strlcat(tbuf, ctx->msg_hmac, data_with_hmac_len);
+
+            ctx->encrypted_msg     = tbuf;
+            ctx->encrypted_msg_len = data_with_hmac_len;
+        }
+    }
+
+    return res;
 }
 
 /* Return the fko SPA encrypted data.
@@ -335,8 +520,8 @@ fko_get_spa_data(fko_ctx_t ctx, char **spa_data)
 
     /* We expect to have encrypted data to process.  If not, we bail.
     */
-    if(ctx->encrypted_msg == NULL
-            || (strnlen(ctx->encrypted_msg, MAX_SPA_ENCRYPTED_SIZE) < 1))
+    if(ctx->encrypted_msg == NULL || ! is_valid_encoded_msg_len(
+                strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE)))
         return(FKO_ERROR_MISSING_ENCODED_DATA);
 
     *spa_data = ctx->encrypted_msg;
@@ -347,9 +532,9 @@ fko_get_spa_data(fko_ctx_t ctx, char **spa_data)
      * prefix
     */
     if(ctx->encryption_type == FKO_ENCRYPTION_RIJNDAEL)
-        *spa_data += strlen(B64_RIJNDAEL_SALT);
+        *spa_data += B64_RIJNDAEL_SALT_STR_LEN;
     else if(ctx->encryption_type == FKO_ENCRYPTION_GPG)
-        *spa_data += strlen(B64_GPG_PREFIX);
+        *spa_data += B64_GPG_PREFIX_STR_LEN;
 
     return(FKO_SUCCESS);
 }
@@ -357,16 +542,28 @@ fko_get_spa_data(fko_ctx_t ctx, char **spa_data)
 /* Set the fko SPA encrypted data.
 */
 int
-fko_set_spa_data(fko_ctx_t ctx, const char *enc_msg)
+fko_set_spa_data(fko_ctx_t ctx, const char * const enc_msg)
 {
+    int         enc_msg_len;
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return FKO_ERROR_CTX_NOT_INITIALIZED;
 
+    enc_msg_len = strnlen(enc_msg, MAX_SPA_ENCODED_MSG_SIZE);
+
+    if(! is_valid_encoded_msg_len(enc_msg_len))
+        return(FKO_ERROR_INVALID_DATA);
+
+    if(ctx->encrypted_msg != NULL)
+        free(ctx->encrypted_msg);
+
     /* First, add the data to the context.
     */
     ctx->encrypted_msg = strdup(enc_msg);
+    ctx->encrypted_msg_len = enc_msg_len;
+
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
