@@ -7,9 +7,9 @@
  *
  * Purpose: Defines various states and flags for libfko operations.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -49,7 +49,9 @@ typedef enum {
     FKO_DIGEST_TYPE_MODIFIED    = 1 << 12,
     FKO_ENCRYPT_TYPE_MODIFIED   = 1 << 13,
     STATE_RESERVED_14           = 1 << 14,
-    FKO_BACKWARD_COMPATIBLE     = 1 << 15
+    FKO_BACKWARD_COMPATIBLE     = 1 << 15,
+    FKO_ENCRYPT_MODE_MODIFIED   = 1 << 16,
+    FKO_HMAC_MODE_MODIFIED      = 1 << 17
 } fko_state_flags_t;
 
 /* This is used in conjunction with the ctx->initial value as a means to
