@@ -7,9 +7,9 @@
  *
  * Purpose: Header file for fwknopd command line options.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -47,6 +47,7 @@ static char *config_map[NUMBER_OF_CONFIG_ENTRIES] = {
     "PCAP_FILTER",
     "PCAP_DISPATCH_COUNT",
     "PCAP_LOOP_SLEEP",
+    "ENABLE_PCAP_ANY_DIRECTION",
     "MAX_SNIFF_BYTES",
     "ENABLE_SPA_PACKET_AGING",
     "MAX_SPA_PACKET_AGE",
@@ -106,6 +107,7 @@ static char *config_map[NUMBER_OF_CONFIG_ENTRIES] = {
 #endif
     "GPG_HOME_DIR",
     "FIREWALL_EXE",
+    "VERBOSE"
 };
 
 
@@ -117,6 +119,7 @@ enum {
     FW_FLUSH,
     GPG_HOME_DIR,
     PCAP_FILE,
+    ENABLE_PCAP_ANY_DIRECTION,
     ROTATE_DIGEST_CACHE,
     NOOP /* Just to be a marker for the end */
 };
@@ -147,6 +150,7 @@ static struct option cmd_opts[] =
     {"override-config",     1, NULL, 'O' },
     {"pcap-file",           1, NULL, PCAP_FILE },
     {"pcap-filter",         1, NULL, 'P'},
+    {"pcap-any-direction",  0, NULL, ENABLE_PCAP_ANY_DIRECTION },
     {"pid-file",            1, NULL, 'p'},
     {"restart",             0, NULL, 'R'},
     {"status",              0, NULL, 'S'},
