@@ -7,9 +7,9 @@
  *
  * Purpose: Command-line and config file processing for fwknop server.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -41,12 +41,16 @@
 static void
 range_check(fko_srv_options_t *opts, char *var, char *val, int low, int high)
 {
-    if (low > atoi(val) || high < atoi(val))
+    int     is_err;
+
+    strtol_wrapper(val, low, high, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
     {
-        fprintf(stderr, "[*] var %s value '%s' not in the range %d-%d\n",
+        log_msg(LOG_ERR, "[*] var %s value '%s' not in the range %d-%d",
             var, val, low, high);
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
+
     return;
 }
 
@@ -62,7 +66,7 @@ set_config_entry(fko_srv_options_t *opts, const int var_ndx, const char *value)
     */
     if(var_ndx < 0 || var_ndx >= NUMBER_OF_CONFIG_ENTRIES)
     {
-        fprintf(stderr, "Index value of %i is not valid\n", var_ndx);
+        log_msg(LOG_ERR, "[*] Index value of %i is not valid", var_ndx);
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
@@ -88,7 +92,7 @@ set_config_entry(fko_srv_options_t *opts, const int var_ndx, const char *value)
 
     if(opts->config[var_ndx] == NULL)
     {
-        fprintf(stderr, "*Fatal memory allocation error!\n");
+        log_msg(LOG_ERR, "[*] Fatal memory allocation error!");
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
@@ -128,6 +132,10 @@ free_configs(fko_srv_options_t *opts)
 static void
 validate_int_var_ranges(fko_srv_options_t *opts)
 {
+#if FIREWALL_IPFW
+    int     is_err = FKO_SUCCESS;
+#endif
+
     range_check(opts, "PCAP_LOOP_SLEEP", opts->config[CONF_PCAP_LOOP_SLEEP],
         1, RCHK_MAX_PCAP_LOOP_SLEEP);
     range_check(opts, "MAX_SPA_PACKET_AGE", opts->config[CONF_MAX_SPA_PACKET_AGE],
@@ -153,13 +161,23 @@ validate_int_var_ranges(fko_srv_options_t *opts)
     /* Make sure the active and expire sets are not identical whenever
      * they are non-zero
     */
-    if((atoi(opts->config[CONF_IPFW_ACTIVE_SET_NUM]) > 0
-            && atoi(opts->config[CONF_IPFW_EXPIRE_SET_NUM]) > 0)
-            && atoi(opts->config[CONF_IPFW_ACTIVE_SET_NUM])
-                == atoi(opts->config[CONF_IPFW_EXPIRE_SET_NUM]))
+    if((strtol_wrapper(opts->config[CONF_IPFW_ACTIVE_SET_NUM],
+                    0, RCHK_MAX_IPFW_SET_NUM, NO_EXIT_UPON_ERR, &is_err) > 0
+            && strtol_wrapper(opts->config[CONF_IPFW_EXPIRE_SET_NUM],
+                0, RCHK_MAX_IPFW_SET_NUM, NO_EXIT_UPON_ERR, &is_err) > 0)
+            && strtol_wrapper(opts->config[CONF_IPFW_ACTIVE_SET_NUM],
+                0, RCHK_MAX_IPFW_SET_NUM, NO_EXIT_UPON_ERR, &is_err)
+                == strtol_wrapper(opts->config[CONF_IPFW_EXPIRE_SET_NUM],
+                    0, RCHK_MAX_IPFW_SET_NUM, NO_EXIT_UPON_ERR, &is_err))
     {
-        fprintf(stderr,
-                "[*] Cannot set identical ipfw active and expire sets.\n");
+        log_msg(LOG_ERR,
+                "[*] Cannot set identical ipfw active and expire sets.");
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    if(is_err != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "[*] invalid integer conversion error.\n");
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
@@ -190,21 +208,24 @@ parse_config_file(fko_srv_options_t *opts, const char *config_file)
 
     struct stat     st;
 
-    /* First see if the config file exists.  If it doesn't, complain
-     * and go on with program defaults.
+    /* Make sure the config file exists.
     */
     if(stat(config_file, &st) != 0)
     {
-        fprintf(stderr, "[*] Config file: '%s' was not found.\n",
+        log_msg(LOG_ERR, "[*] Config file: '%s' was not found.",
             config_file);
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
-    verify_file_perms_ownership(config_file);
+    if(verify_file_perms_ownership(config_file) != 1)
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
 
+    /* See the comment in the parse_access_file() function regarding security
+     * here relative to a TOCTOU bug flagged by Coverity.
+    */
     if ((cfile_ptr = fopen(config_file, "r")) == NULL)
     {
-        fprintf(stderr, "[*] Could not open config file: %s\n",
+        log_msg(LOG_ERR, "[*] Could not open config file: %s",
             config_file);
         perror(NULL);
 
@@ -224,7 +245,7 @@ parse_config_file(fko_srv_options_t *opts, const char *config_file)
 
         if(sscanf(conf_line_buf, "%s %[^;\n\r]", var, val) != 2)
         {
-            fprintf(stderr,
+            log_msg(LOG_ERR,
                 "*Invalid config file entry in %s at line %i.\n - '%s'",
                 config_file, numLines, conf_line_buf
             );
@@ -253,8 +274,8 @@ parse_config_file(fko_srv_options_t *opts, const char *config_file)
                     {
                         if((cndx = config_entry_index(opts, tmp1)) >= 0)
                         {
-                            strlcpy(val, opts->config[cndx], MAX_LINE_LEN);
-                            strlcat(val, tmp2, MAX_LINE_LEN);
+                            strlcpy(val, opts->config[cndx], sizeof(val));
+                            strlcat(val, tmp2, sizeof(val));
                         }
                     }
                 }
@@ -266,8 +287,8 @@ parse_config_file(fko_srv_options_t *opts, const char *config_file)
         }
 
         if(good_ent == 0)
-            fprintf(stderr,
-                "*Ignoring unknown configuration parameter: '%s' in %s\n",
+            log_msg(LOG_ERR,
+                "[*] Ignoring unknown configuration parameter: '%s' in %s",
                 var, config_file
             );
     }
@@ -282,7 +303,7 @@ parse_config_file(fko_srv_options_t *opts, const char *config_file)
 static void
 validate_options(fko_srv_options_t *opts)
 {
-    char tmp_path[MAX_PATH_LEN];
+    char tmp_path[MAX_PATH_LEN] = {0};
 
     /* If no conf dir is set in the config file, use the default.
     */
@@ -304,12 +325,12 @@ validate_options(fko_srv_options_t *opts)
 
     if(opts->config[CONF_FWKNOP_PID_FILE] == NULL)
     {
-        strlcpy(tmp_path, opts->config[CONF_FWKNOP_RUN_DIR], MAX_PATH_LEN);
+        strlcpy(tmp_path, opts->config[CONF_FWKNOP_RUN_DIR], sizeof(tmp_path));
 
         if(tmp_path[strlen(tmp_path)-1] != '/')
-            strlcat(tmp_path, "/", MAX_PATH_LEN);
+            strlcat(tmp_path, "/", sizeof(tmp_path));
 
-        strlcat(tmp_path, DEF_PID_FILENAME, MAX_PATH_LEN);
+        strlcat(tmp_path, DEF_PID_FILENAME, sizeof(tmp_path));
 
         set_config_entry(opts, CONF_FWKNOP_PID_FILE, tmp_path);
     }
@@ -320,17 +341,17 @@ validate_options(fko_srv_options_t *opts)
     if(opts->config[CONF_DIGEST_DB_FILE] == NULL)
 #endif
     {
-        strlcpy(tmp_path, opts->config[CONF_FWKNOP_RUN_DIR], MAX_PATH_LEN);
+        strlcpy(tmp_path, opts->config[CONF_FWKNOP_RUN_DIR], sizeof(tmp_path));
 
         if(tmp_path[strlen(tmp_path)-1] != '/')
-            strlcat(tmp_path, "/", MAX_PATH_LEN);
+            strlcat(tmp_path, "/", sizeof(tmp_path));
 
 
 #if USE_FILE_CACHE
-        strlcat(tmp_path, DEF_DIGEST_CACHE_FILENAME, MAX_PATH_LEN);
+        strlcat(tmp_path, DEF_DIGEST_CACHE_FILENAME, sizeof(tmp_path));
         set_config_entry(opts, CONF_DIGEST_FILE, tmp_path);
 #else
-        strlcat(tmp_path, DEF_DIGEST_CACHE_DB_FILENAME, MAX_PATH_LEN);
+        strlcat(tmp_path, DEF_DIGEST_CACHE_DB_FILENAME, sizeof(tmp_path));
         set_config_entry(opts, CONF_DIGEST_DB_FILE, tmp_path);
 #endif
     }
@@ -591,16 +612,16 @@ validate_options(fko_srv_options_t *opts)
 
     if((opts->dump_config + opts->kill + opts->restart + opts->status) > 1)
     {
-        fprintf(stderr,
-            "The -D, -K, -R, and -S options are mutually exclusive.  Pick only one.\n"
+        log_msg(LOG_ERR,
+            "The -D, -K, -R, and -S options are mutually exclusive.  Pick only one."
         );
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
     if(opts->config[CONF_FIREWALL_EXE] == NULL)
     {
-        fprintf(stderr,
-            "No firewall command executable is set. Please check FIREWALL_EXE in fwknopd.conf.\n"
+        log_msg(LOG_ERR,
+            "[*] No firewall command executable is set. Please check FIREWALL_EXE in fwknopd.conf."
         );
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
@@ -631,10 +652,10 @@ set_preconfig_entries(fko_srv_options_t *opts)
 void
 config_init(fko_srv_options_t *opts, int argc, char **argv)
 {
-    int             cmd_arg, index;
+    int             cmd_arg, index, is_err;
     unsigned char   got_conf_file = 0, got_override_config = 0;
 
-    char            override_file[MAX_LINE_LEN];
+    char            override_file[MAX_LINE_LEN] = {0};
     char           *ndx, *cmrk;
 
     /* Zero out options and opts_track.
@@ -665,27 +686,27 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
                 clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
                 break;
 
-        /* Look for configuration file arg.
-        */
-        case 'c':
-            set_config_entry(opts, CONF_CONFIG_FILE, optarg);
-            got_conf_file++;
-
-            /* If we already have the config_override option, we are done.
+            /* Look for configuration file arg.
             */
-            if(got_override_config > 0)
-                break;
+            case 'c':
+                set_config_entry(opts, CONF_CONFIG_FILE, optarg);
+                got_conf_file++;
 
-        /* Look for override configuration file arg.
-        */
-        case 'O':
-            set_config_entry(opts, CONF_OVERRIDE_CONFIG, optarg);
-            got_override_config++;
+                /* If we already have the config_override option, we are done.
+                */
+                if(got_override_config > 0)
+                    break;
 
-            /* If we already have the conf_file option, we are done.
+            /* Look for override configuration file arg.
             */
-            if(got_conf_file > 0)
-                break;
+            case 'O':
+                set_config_entry(opts, CONF_OVERRIDE_CONFIG, optarg);
+                got_override_config++;
+
+                /* If we already have the conf_file option, we are done.
+                */
+                if(got_conf_file > 0)
+                    break;
         }
     }
 
@@ -707,7 +728,7 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
     {
         /* Make a copy of the override_config string so we can munge it.
         */
-        strlcpy(override_file, opts->config[CONF_OVERRIDE_CONFIG], MAX_LINE_LEN);
+        strlcpy(override_file, opts->config[CONF_OVERRIDE_CONFIG], sizeof(override_file));
 
         ndx  = override_file;
         cmrk = strchr(ndx, ',');
@@ -735,6 +756,20 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
         }
     }
 
+    /* Set up the verbosity level according to the value found in the
+     * config files */
+    if (opts->config[CONF_VERBOSE] != NULL)
+    {
+        opts->verbose = strtol_wrapper(opts->config[CONF_VERBOSE], 0, -1,
+                                       NO_EXIT_UPON_ERR, &is_err);
+        if(is_err != FKO_SUCCESS)
+        {
+            log_msg(LOG_ERR, "[*] VERBOSE value '%s' not in the range (>0)",
+                opts->config[CONF_VERBOSE]);
+            clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+        }
+    }
+
     /* Reset the options index so we can run through them again.
     */
     optind = 0;
@@ -753,7 +788,15 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
                 /* This was handled earlier */
                 break;
             case 'C':
-                opts->packet_ctr_limit = atoi(optarg);
+                opts->packet_ctr_limit = strtol_wrapper(optarg,
+                        0, (2 << 30), NO_EXIT_UPON_ERR, &is_err);
+                if(is_err != FKO_SUCCESS)
+                {
+                    log_msg(LOG_ERR,
+                        "[*] invalid -C packet count limit '%s'",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
                 break;
             case 'd':
 #if USE_FILE_CACHE
@@ -785,8 +828,8 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
                 }
                 else
                 {
-                    fprintf(stderr,
-                        "[*] Directory '%s' could not stat()/does not exist?\n",
+                    log_msg(LOG_ERR,
+                        "[*] Directory '%s' could not stat()/does not exist?",
                         optarg);
                     clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
                 }
@@ -812,6 +855,9 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case PCAP_FILE:
                 set_config_entry(opts, CONF_PCAP_FILE, optarg);
                 break;
+            case ENABLE_PCAP_ANY_DIRECTION:
+                opts->pcap_any_direction = 1;
+                break;
             case ROTATE_DIGEST_CACHE:
                 opts->rotate_digest_cache = 1;
                 break;
@@ -821,6 +867,7 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'S':
                 opts->status = 1;
                 break;
+            /* Verbosity level */
             case 'v':
                 opts->verbose++;
                 break;
@@ -867,7 +914,8 @@ dump_config(const fko_srv_options_t *opts)
 void
 usage(void)
 {
-    fprintf(stdout, "\n%s server version %s\n%s\n\n", MY_NAME, MY_VERSION, MY_DESC);
+    fprintf(stdout, "\n%s server version %s\n%s - http://www.cipherdyne.org/fwknop/\n\n",
+            MY_NAME, MY_VERSION, MY_DESC);
     fprintf(stdout,
       "Usage: fwknopd [options]\n\n"
       " -h, --help              - Print this usage message and exit.\n"
