@@ -7,9 +7,9 @@
  *
  * Purpose: An implementation of an fwknop server.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -42,16 +42,17 @@
 
 /* Prototypes
 */
-static void check_dir_path(const char *path, const char *path_name, const unsigned char use_basename);
-static int make_dir_path(const char *path);
-static void daemonize_process(fko_srv_options_t *opts);
+static void check_dir_path(const char * const path,
+        const char * const path_name, const unsigned char use_basename);
+static int make_dir_path(const char * const path);
+static void daemonize_process(fko_srv_options_t * const opts);
 static int write_pid_file(fko_srv_options_t *opts);
 static pid_t get_running_pid(const fko_srv_options_t *opts);
 
 int
 main(int argc, char **argv)
 {
-    int                 res, last_sig, rp_cache_count;
+    int                 res, last_sig, rp_cache_count, is_err;
     char               *locale;
     pid_t               old_pid;
 
@@ -100,11 +101,15 @@ main(int argc, char **argv)
             old_pid = write_pid_file(&opts);
 
             if(old_pid > 0)
+            {
                 fprintf(stdout, "Detected fwknopd is running (pid=%i).\n", old_pid);
+                clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+            }
             else
+            {
                 fprintf(stdout, "No running fwknopd detected.\n");
-
-            clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+                clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
+            }
         }
 
         /* Restart the currently running fwknopd?
@@ -138,6 +143,9 @@ main(int argc, char **argv)
         */
         init_logging(&opts);
 
+        /* Update the verbosity level for the log module */
+        log_set_verbosity(LOG_DEFAULT_VERBOSITY + opts.verbose);
+
 #if HAVE_LOCALE_H
         /* Set the locale if specified.
         */
@@ -155,10 +163,9 @@ main(int argc, char **argv)
             }
             else
             {
-                if(opts.verbose)
-                    log_msg(LOG_INFO,
-                        "Locale set to '%s'.", opts.config[CONF_LOCALE]
-                    );
+                log_msg(LOG_INFO,
+                    "Locale set to '%s'.", opts.config[CONF_LOCALE]
+                );
             }
         }
 #endif
@@ -174,7 +181,8 @@ main(int argc, char **argv)
          * via fw_rules_dump() in --fw-list mode before changing around any rules
          * of an existing fwknopd process.
         */
-        fw_config_init(&opts);
+        if(fw_config_init(&opts) != 1)
+            clean_exit(&opts, FW_CLEANUP, EXIT_FAILURE);
 
         if(opts.fw_list == 1 || opts.fw_list_all == 1)
         {
@@ -208,7 +216,7 @@ main(int argc, char **argv)
         */
         if(get_running_pid(&opts) != getpid())
         {
-            /* If foreground mode is not set, the fork off and become a daemon.
+            /* If foreground mode is not set, then fork off and become a daemon.
             * Otherwise, attempt to get the pid file lock and go on.
             */
             if(opts.foreground == 0)
@@ -221,14 +229,14 @@ main(int argc, char **argv)
                 if(old_pid > 0)
                 {
                     fprintf(stderr,
-                        "* An instance of fwknopd is already running: (PID=%i).\n", old_pid
+                        "[*] An instance of fwknopd is already running: (PID=%i).\n", old_pid
                     );
 
                     clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
                 }
                 else if(old_pid < 0)
                 {
-                    fprintf(stderr, "* PID file error. The lock may not be effective.\n");
+                    fprintf(stderr, "[*] PID file error. The lock may not be effective.\n");
                 }
             }
 
@@ -258,6 +266,10 @@ main(int argc, char **argv)
                 log_msg(LOG_WARNING,
                     "Error opening digest cache file. Incoming digests will not be remembered."
                 );
+                /* Destination points to heap memory, and is guaranteed to be
+                 * at least two bytes large via validate_options(),
+                 * DEF_ENABLE_DIGEST_PERSISTENCE, and set_config_entry()
+                */
                 strlcpy(opts.config[CONF_ENABLE_DIGEST_PERSISTENCE], "N", 2);
             }
 
@@ -275,23 +287,26 @@ main(int argc, char **argv)
         /* Prepare the firewall - i.e. flush any old rules and (for iptables)
          * create fwknop chains.
         */
-        fw_initialize(&opts);
+        if(fw_initialize(&opts) != 1)
+            clean_exit(&opts, FW_CLEANUP, EXIT_FAILURE);
 
         /* If the TCP server option was set, fire it up here.
         */
         if(strncasecmp(opts.config[CONF_ENABLE_TCP_SERVER], "Y", 1) == 0)
         {
-            if(atoi(opts.config[CONF_TCPSERV_PORT]) <= 0
-              || atoi(opts.config[CONF_TCPSERV_PORT]) > MAX_PORT)
+
+            strtol_wrapper(opts.config[CONF_TCPSERV_PORT],
+                    1, MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
+            if(is_err == FKO_SUCCESS)
+            {
+                run_tcp_server(&opts);
+            }
+            else
             {
                 log_msg(LOG_WARNING,
                     "WARNING: ENABLE_TCP_SERVER is set, but TCPSERV_PORT is not valid. TCP server not started!"
                 );
             }
-            else
-            {
-                run_tcp_server(&opts);
-            }
         }
 
         /* Intiate pcap capture mode...
@@ -381,7 +396,7 @@ main(int argc, char **argv)
 /* Ensure the specified directory exists.  If not, create it or die.
 */
 static void
-check_dir_path(const char *filepath, const char *fp_desc, const unsigned char use_basename)
+check_dir_path(const char * const filepath, const char * const fp_desc, const unsigned char use_basename)
 {
     struct stat     st;
     int             res = 0;
@@ -407,7 +422,7 @@ check_dir_path(const char *filepath, const char *fp_desc, const unsigned char us
     if(use_basename && ((ndx = strrchr(filepath, PATH_SEP)) != NULL))
         strlcpy(tmp_path, filepath, (ndx-filepath)+1);
     else
-        strlcpy(tmp_path, filepath, MAX_PATH_LEN);
+        strlcpy(tmp_path, filepath, sizeof(tmp_path));
 
     /* At this point, we should make the path is more than just the
      * PATH_SEP.  If it is not, silently return.
@@ -458,7 +473,7 @@ check_dir_path(const char *filepath, const char *fp_desc, const unsigned char us
         if(! S_ISDIR(st.st_mode))
         {
             log_msg(LOG_ERR,
-                "Specified %s directory: %s is NOT a directory\n\n", fp_desc, tmp_path
+                "Specified %s directory: %s is NOT a directory", fp_desc, tmp_path
             );
             exit(EXIT_FAILURE);
         }
@@ -466,14 +481,14 @@ check_dir_path(const char *filepath, const char *fp_desc, const unsigned char us
 }
 
 static int
-make_dir_path(const char *run_dir)
+make_dir_path(const char * const run_dir)
 {
     struct stat     st;
     int             res = 0, len = 0;
     char            tmp_path[MAX_PATH_LEN];
     char            *ndx;
 
-    strlcpy(tmp_path, run_dir, MAX_PATH_LEN);
+    strlcpy(tmp_path, run_dir, sizeof(tmp_path));
 
     len = strlen(tmp_path);
 
@@ -507,7 +522,7 @@ make_dir_path(const char *run_dir)
                     if(stat(tmp_path, &st) != 0)
                     {
                         log_msg(LOG_ERR,
-                            "Could not create component: %s of %s\n\n", tmp_path, run_dir
+                            "Could not create component: %s of %s", tmp_path, run_dir
                         );
                         return(ENOTDIR);
                     }
@@ -517,7 +532,7 @@ make_dir_path(const char *run_dir)
             if(! S_ISDIR(st.st_mode))
             {
                 log_msg(LOG_ERR,
-                    "Component: %s of %s is NOT a directory\n\n", tmp_path, run_dir
+                    "Component: %s of %s is NOT a directory", tmp_path, run_dir
                 );
                 return(ENOTDIR);
             }
@@ -535,7 +550,7 @@ make_dir_path(const char *run_dir)
  * and close unneeded standard filehandles.
 */
 static void
-daemonize_process(fko_srv_options_t *opts)
+daemonize_process(fko_srv_options_t * const opts)
 {
     pid_t pid, old_pid;
 
@@ -565,14 +580,15 @@ daemonize_process(fko_srv_options_t *opts)
     if(old_pid > 0)
     {
         fprintf(stderr,
-            "* An instance of fwknopd is already running: (PID=%i).\n", old_pid
+            "[*] An instance of fwknopd is already running: (PID=%i).\n", old_pid
         );
 
         exit(EXIT_FAILURE);
     }
     else if(old_pid < 0)
     {
-        fprintf(stderr, "* PID file error. The lock may not be effective.\n");
+        fprintf(stderr,
+                "[*] PID file error. The lock may not be effective.\n");
     }
 
     /* Chdir to the root of the filesystem
@@ -614,7 +630,12 @@ write_pid_file(fko_srv_options_t *opts)
         return -1;
     }
 
-    fcntl(op_fd, F_SETFD, FD_CLOEXEC);
+    if(fcntl(op_fd, F_SETFD, FD_CLOEXEC) == -1)
+    {
+        close(op_fd);
+        perror("Unexpected error from fcntl: ");
+        return -1;
+    }
 
     /* Attempt to lock the PID file.  If we get an EWOULDBLOCK
      * error, another instance already has the lock. So we grab
@@ -623,14 +644,14 @@ write_pid_file(fko_srv_options_t *opts)
     lck_res = lockf(op_fd, F_TLOCK, 0);
     if(lck_res == -1)
     {
+        close(op_fd);
+
         if(errno != EAGAIN)
         {
             perror("Unexpected error from lockf: ");
             return -1;
         }
 
-        close(op_fd);
-
         /* Look for an existing lock holder. If we get a pid return it.
         */
         old_pid = get_running_pid(opts);
@@ -648,9 +669,8 @@ write_pid_file(fko_srv_options_t *opts)
     my_pid = getpid();
     snprintf(buf, PID_BUFLEN, "%i\n", my_pid);
 
-    if(opts->verbose > 1)
-        log_msg(LOG_INFO, "[+] Writing my PID (%i) to the lock file: %s\n",
-            my_pid, opts->config[CONF_FWKNOP_PID_FILE]);
+    log_msg(LOG_DEBUG, "[+] Writing my PID (%i) to the lock file: %s",
+        my_pid, opts->config[CONF_FWKNOP_PID_FILE]);
 
     num_bytes = write(op_fd, buf, strlen(buf));
 
@@ -672,26 +692,39 @@ write_pid_file(fko_srv_options_t *opts)
 static pid_t
 get_running_pid(const fko_srv_options_t *opts)
 {
-    int     op_fd;
+    int     op_fd, is_err;
     char    buf[PID_BUFLEN] = {0};
     pid_t   rpid            = 0;
 
 
-    verify_file_perms_ownership(opts->config[CONF_FWKNOP_PID_FILE]);
+    if(verify_file_perms_ownership(opts->config[CONF_FWKNOP_PID_FILE]) != 1)
+    {
+        fprintf(stderr, "verify_file_perms_ownership() error\n");
+        return(rpid);
+    }
 
     op_fd = open(opts->config[CONF_FWKNOP_PID_FILE], O_RDONLY);
 
-    if(op_fd > 0)
+    if(op_fd == -1)
     {
-        if (read(op_fd, buf, PID_BUFLEN) > 0)
-        {
-            buf[PID_BUFLEN-1] = '\0';
-            rpid = (pid_t)atoi(buf);
-        }
-
-        close(op_fd);
+        if((opts->foreground != 0) && (opts->verbose != 0))
+            perror("Error trying to open PID file: ");
+        return(rpid);
     }
 
+    if (read(op_fd, buf, PID_BUFLEN) > 0)
+    {
+        buf[PID_BUFLEN-1] = '\0';
+        /* max pid value is configurable on Linux
+        */
+        rpid = (pid_t) strtol_wrapper(buf, 0, (2 << 30),
+                NO_EXIT_UPON_ERR, &is_err);
+        if(is_err != FKO_SUCCESS)
+            rpid = 0;
+    }
+
+    close(op_fd);
+
     return(rpid);
 }
 
