@@ -7,9 +7,9 @@
  *
  * Purpose: Process an incoming SPA data packet for fwknopd.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -62,6 +62,15 @@ preprocess_spa_data(fko_srv_options_t *opts, const char *src_ip)
     */
     spa_pkt->packet_data_len = 0;
 
+    /* These two checks are already done in process_packet(), but this is a
+     * defensive measure to run them again here
+    */
+    if(pkt_data_len < MIN_SPA_DATA_SIZE)
+        return(SPA_MSG_BAD_DATA);
+
+    if(pkt_data_len > MAX_SPA_PACKET_LEN)
+        return(SPA_MSG_BAD_DATA);
+
     /* Ignore any SPA packets that contain the Rijndael or GnuPG prefixes
      * since an attacker might have tacked them on to a previously seen
      * SPA packet in an attempt to get past the replay check.  And, we're
@@ -69,11 +78,11 @@ preprocess_spa_data(fko_srv_options_t *opts, const char *src_ip)
      * a prefix after the outer one is stripped off won't decrypt properly
      * anyway because libfko would not add a new one.
     */
-    if(strncmp(ndx, B64_RIJNDAEL_SALT, B64_RIJNDAEL_SALT_STR_LEN) == 0)
+    if(constant_runtime_cmp(ndx, B64_RIJNDAEL_SALT, B64_RIJNDAEL_SALT_STR_LEN) == 0)
         return(SPA_MSG_BAD_DATA);
 
     if(pkt_data_len > MIN_GNUPG_MSG_SIZE
-            && strncmp(ndx, B64_GPG_PREFIX, B64_GPG_PREFIX_STR_LEN) == 0)
+            && constant_runtime_cmp(ndx, B64_GPG_PREFIX, B64_GPG_PREFIX_STR_LEN) == 0)
         return(SPA_MSG_BAD_DATA);
 
     /* Detect and parse out SPA data from an HTTP request. If the SPA data
@@ -136,12 +145,15 @@ get_raw_digest(char **digest, char *pkt_data)
     /* initialize an FKO context with no decryption key just so
      * we can get the outer message digest
     */
-    res = fko_new_with_data(&ctx, (char *)pkt_data, NULL);
+    res = fko_new_with_data(&ctx, (char *)pkt_data, NULL, 0,
+            FKO_DEFAULT_ENC_MODE, NULL, 0, 0);
+
     if(res != FKO_SUCCESS)
     {
         log_msg(LOG_WARNING, "Error initializing FKO context from SPA data: %s",
             fko_errstr(res));
         fko_destroy(ctx);
+        ctx = NULL;
         return(SPA_MSG_FKO_CTX_ERROR);
     }
 
@@ -151,6 +163,7 @@ get_raw_digest(char **digest, char *pkt_data)
         log_msg(LOG_WARNING, "Error setting digest type for SPA data: %s",
             fko_errstr(res));
         fko_destroy(ctx);
+        ctx = NULL;
         return(SPA_MSG_DIGEST_ERROR);
     }
 
@@ -160,6 +173,7 @@ get_raw_digest(char **digest, char *pkt_data)
         log_msg(LOG_WARNING, "Error setting digest for SPA data: %s",
             fko_errstr(res));
         fko_destroy(ctx);
+        ctx = NULL;
         return(SPA_MSG_DIGEST_ERROR);
     }
 
@@ -169,15 +183,17 @@ get_raw_digest(char **digest, char *pkt_data)
         log_msg(LOG_WARNING, "Error getting digest from SPA data: %s",
             fko_errstr(res));
         fko_destroy(ctx);
+        ctx = NULL;
         return(SPA_MSG_DIGEST_ERROR);
     }
 
     *digest = strdup(tmp_digest);
 
-    if (digest == NULL)
-        return SPA_MSG_ERROR;
+    if (*digest == NULL)
+        res = SPA_MSG_ERROR;  /* really a strdup() memory allocation problem */
 
     fko_destroy(ctx);
+    ctx = NULL;
 
     return res;
 }
@@ -255,6 +271,8 @@ incoming_spa(fko_srv_options_t *opts)
     time_t          now_ts;
     int             res, status, ts_diff, enc_type, stanza_num=0;
     int             added_replay_digest = 0, pkt_data_len=0;
+    int             is_err, cmd_exec_success = 0, attempted_decrypt = 0;
+    int             conf_pkt_age = 0;
 
     spa_pkt_info_t *spa_pkt = &(opts->spa_pkt);
 
@@ -277,9 +295,8 @@ incoming_spa(fko_srv_options_t *opts)
     res = preprocess_spa_data(opts, spadat.pkt_source_ip);
     if(res != FKO_SUCCESS)
     {
-        if(opts->verbose > 1)
-            log_msg(LOG_INFO, "preprocess_spa_data() returned error %i: '%s' for incoming packet.",
-                res, get_errstr(res));
+        log_msg(LOG_DEBUG, "[%s] preprocess_spa_data() returned error %i: '%s' for incoming packet.",
+            spadat.pkt_source_ip, res, get_errstr(res));
         return;
     }
 
@@ -289,6 +306,17 @@ incoming_spa(fko_srv_options_t *opts)
         hex_dump(spa_pkt->packet_data, pkt_data_len);
     }
 
+    if(strncasecmp(opts->config[CONF_ENABLE_SPA_PACKET_AGING], "Y", 1) == 0)
+    {
+        conf_pkt_age = strtol_wrapper(opts->config[CONF_MAX_SPA_PACKET_AGE],
+                0, RCHK_MAX_SPA_PACKET_AGE, NO_EXIT_UPON_ERR, &is_err);
+        if(is_err != FKO_SUCCESS)
+        {
+            log_msg(LOG_ERR, "[*] [%s] invalid MAX_SPA_PACKET_AGE", spadat.pkt_source_ip);
+            return;
+        }
+    }
+
     if (is_src_match(opts->acc_stanzas, ntohl(spa_pkt->packet_src_ip)))
     {
         if(strncasecmp(opts->config[CONF_ENABLE_DIGEST_PERSISTENCE], "Y", 1) == 0)
@@ -326,8 +354,23 @@ incoming_spa(fko_srv_options_t *opts)
     */
     while(acc)
     {
+        res = FKO_SUCCESS;
+        cmd_exec_success  = 0;
+        attempted_decrypt = 0;
         stanza_num++;
 
+        /* Start access loop with a clean FKO context
+        */
+        if(ctx != NULL)
+        {
+            if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
+                log_msg(LOG_WARNING,
+                    "[%s] (stanza #%d) fko_destroy() could not zero out sensitive data buffer.",
+                    spadat.pkt_source_ip, stanza_num, fko_errstr(res)
+                );
+            ctx = NULL;
+        }
+
         /* Check for a match for the SPA source IP and the access stanza
         */
         if(! compare_addr_list(acc->source_list, ntohl(spa_pkt->packet_src_ip)))
@@ -339,8 +382,7 @@ incoming_spa(fko_srv_options_t *opts)
         log_msg(LOG_INFO, "(stanza #%d) SPA Packet from IP: %s received with access source match",
             stanza_num, spadat.pkt_source_ip);
 
-        if(opts->verbose > 1)
-            log_msg(LOG_INFO, "SPA Packet: '%s'\n", spa_pkt->packet_data);
+        log_msg(LOG_DEBUG, "SPA Packet: '%s'", spa_pkt->packet_data);
 
         /* Make sure this access stanza has not expired
         */
@@ -355,8 +397,8 @@ incoming_spa(fko_srv_options_t *opts)
             {
                 if(time(NULL) > acc->access_expire_time)
                 {
-                    log_msg(LOG_INFO, "(stanza #%d) Access stanza has expired",
-                        stanza_num);
+                    log_msg(LOG_INFO, "[%s] (stanza #%d) Access stanza has expired",
+                        spadat.pkt_source_ip, stanza_num);
                     acc->expired = 1;
                     acc = acc->next;
                     continue;
@@ -369,36 +411,48 @@ incoming_spa(fko_srv_options_t *opts)
         */
         enc_type = fko_encryption_type((char *)spa_pkt->packet_data);
 
-        if(acc->use_rijndael && enc_type == FKO_ENCRYPTION_RIJNDAEL)
+        if(acc->use_rijndael)
         {
-            if(acc->key != NULL)
-                res = fko_new_with_data(&ctx, (char *)spa_pkt->packet_data, acc->key);
-            else
+            if (acc->key == NULL)
             {
                 log_msg(LOG_ERR,
-                    "(stanza #%d) No KEY for RIJNDAEL encrypted messages",
-                    stanza_num
+                    "[%s] (stanza #%d) No KEY for RIJNDAEL encrypted messages",
+                    spadat.pkt_source_ip, stanza_num
                 );
                 acc = acc->next;
                 continue;
             }
+
+            /* Command mode messages may be quite long
+            */
+            if(acc->enable_cmd_exec || enc_type == FKO_ENCRYPTION_RIJNDAEL)
+            {
+                res = fko_new_with_data(&ctx, (char *)spa_pkt->packet_data,
+                    acc->key, acc->key_len, acc->encryption_mode, acc->hmac_key,
+                    acc->hmac_key_len, acc->hmac_type);
+                attempted_decrypt = 1;
+                if(res == FKO_SUCCESS)
+                    cmd_exec_success = 1;
+            }
         }
-        else if(acc->use_gpg && enc_type == FKO_ENCRYPTION_GPG)
+
+        if(acc->use_gpg && enc_type == FKO_ENCRYPTION_GPG && cmd_exec_success == 0)
         {
             /* For GPG we create the new context without decrypting on the fly
-             * so we can set some  GPG parameters first.
+             * so we can set some GPG parameters first.
             */
             if(acc->gpg_decrypt_pw != NULL || acc->gpg_allow_no_pw)
             {
-                res = fko_new_with_data(&ctx, (char *)spa_pkt->packet_data, NULL);
+                res = fko_new_with_data(&ctx, (char *)spa_pkt->packet_data, NULL,
+                        0, FKO_ENC_MODE_ASYMMETRIC, acc->hmac_key,
+                        acc->hmac_key_len, acc->hmac_type);
+
                 if(res != FKO_SUCCESS)
                 {
                     log_msg(LOG_WARNING,
-                        "(stanza #%d) Error creating fko context (before decryption): %s",
-                        stanza_num, fko_errstr(res)
+                        "[%s] (stanza #%d) Error creating fko context (before decryption): %s",
+                        spadat.pkt_source_ip, stanza_num, fko_errstr(res)
                     );
-                    if(ctx != NULL)
-                        fko_destroy(ctx);
                     acc = acc->next;
                     continue;
                 }
@@ -406,18 +460,18 @@ incoming_spa(fko_srv_options_t *opts)
                 /* Set whatever GPG parameters we have.
                 */
                 if(acc->gpg_home_dir != NULL)
+                {
                     res = fko_set_gpg_home_dir(ctx, acc->gpg_home_dir);
                     if(res != FKO_SUCCESS)
                     {
                         log_msg(LOG_WARNING,
-                            "(stanza #%d) Error setting GPG keyring path to %s: %s",
-                            stanza_num, acc->gpg_home_dir, fko_errstr(res)
+                            "[%s] (stanza #%d) Error setting GPG keyring path to %s: %s",
+                            spadat.pkt_source_ip, stanza_num, acc->gpg_home_dir, fko_errstr(res)
                         );
-                        if(ctx != NULL)
-                            fko_destroy(ctx);
                         acc = acc->next;
                         continue;
                     }
+                }
 
                 if(acc->gpg_decrypt_id != NULL)
                     fko_set_gpg_recipient(ctx, acc->gpg_decrypt_id);
@@ -443,11 +497,12 @@ incoming_spa(fko_srv_options_t *opts)
 
                 /* Now decrypt the data.
                 */
-                res = fko_decrypt_spa_data(ctx, acc->gpg_decrypt_pw);
-
+                res = fko_decrypt_spa_data(ctx, acc->gpg_decrypt_pw, 0);
+                attempted_decrypt = 1;
             }
         }
-        else
+
+        if(attempted_decrypt == 0)
         {
             log_msg(LOG_ERR,
                 "(stanza #%d) No stanza encryption mode match for encryption type: %i.",
@@ -460,30 +515,26 @@ incoming_spa(fko_srv_options_t *opts)
         */
         if(res != FKO_SUCCESS)
         {
-            log_msg(LOG_WARNING, "(stanza #%d) Error creating fko context: %s",
-                stanza_num, fko_errstr(res));
+            log_msg(LOG_WARNING, "[%s] (stanza #%d) Error creating fko context: %s",
+                spadat.pkt_source_ip, stanza_num, fko_errstr(res));
 
             if(IS_GPG_ERROR(res))
-                log_msg(LOG_WARNING, "(stanza #%d) - GPG ERROR: %s",
-                    stanza_num, fko_gpg_errstr(ctx));
+                log_msg(LOG_WARNING, "[%s] (stanza #%d) - GPG ERROR: %s",
+                    spadat.pkt_source_ip, stanza_num, fko_gpg_errstr(ctx));
 
-            if(ctx != NULL)
-                fko_destroy(ctx);
             acc = acc->next;
             continue;
         }
 
         /* Add this SPA packet into the replay detection cache
         */
-        if (! added_replay_digest)
+        if (added_replay_digest == 0)
         {
             res = add_replay(opts, raw_digest);
             if (res != SPA_MSG_SUCCESS)
             {
-                log_msg(LOG_WARNING, "(stanza #%d) Could not add digest to replay cache",
-                    stanza_num);
-                if(ctx != NULL)
-                    fko_destroy(ctx);
+                log_msg(LOG_WARNING, "[%s] (stanza #%d) Could not add digest to replay cache",
+                    spadat.pkt_source_ip, stanza_num);
                 acc = acc->next;
                 continue;
             }
@@ -493,9 +544,8 @@ incoming_spa(fko_srv_options_t *opts)
         /* At this point, we assume the SPA data is valid.  Now we need to see
          * if it meets our access criteria.
         */
-        if(opts->verbose > 1)
-            log_msg(LOG_INFO, "(stanza #%d) SPA Decode (res=%i):\n%s",
-                stanza_num, res, dump_ctx(ctx));
+        log_msg(LOG_DEBUG, "[%s] (stanza #%d) SPA Decode (res=%i):\n%s",
+            spadat.pkt_source_ip, stanza_num, res, dump_ctx(ctx));
 
         /* First, if this is a GPG message, and GPG_REMOTE_ID list is not empty,
          * then we need to make sure this incoming message is signer ID matches
@@ -506,25 +556,21 @@ incoming_spa(fko_srv_options_t *opts)
             res = fko_get_gpg_signature_id(ctx, &gpg_id);
             if(res != FKO_SUCCESS)
             {
-                log_msg(LOG_WARNING, "(stanza #%d) Error pulling the GPG signature ID from the context: %s",
-                    stanza_num, fko_gpg_errstr(ctx));
-                if(ctx != NULL)
-                    fko_destroy(ctx);
+                log_msg(LOG_WARNING,
+                    "[%s] (stanza #%d) Error pulling the GPG signature ID from the context: %s",
+                    spadat.pkt_source_ip, stanza_num, fko_gpg_errstr(ctx));
                 acc = acc->next;
                 continue;
             }
 
-            if(opts->verbose)
-                log_msg(LOG_INFO, "(stanza #%d) Incoming SPA data signed by '%s'.",
-                    stanza_num, gpg_id);
+            log_msg(LOG_INFO, "[%s] (stanza #%d) Incoming SPA data signed by '%s'.",
+                spadat.pkt_source_ip, stanza_num, gpg_id);
 
             if(acc->gpg_remote_id != NULL && !acc_check_gpg_remote_id(acc, gpg_id))
             {
                 log_msg(LOG_WARNING,
-                    "(stanza #%d) Incoming SPA packet signed by ID: %s, but that ID is not the GPG_REMOTE_ID list.",
-                    stanza_num, gpg_id);
-                if(ctx != NULL)
-                    fko_destroy(ctx);
+                    "[%s] (stanza #%d) Incoming SPA packet signed by ID: %s, but that ID is not the GPG_REMOTE_ID list.",
+                    spadat.pkt_source_ip, stanza_num, gpg_id);
                 acc = acc->next;
                 continue;
             }
@@ -547,11 +593,9 @@ incoming_spa(fko_srv_options_t *opts)
 
         if(res != FKO_SUCCESS)
         {
-            log_msg(LOG_ERR, "(stanza #%d) Unexpected error pulling SPA data from the context: %s",
-                stanza_num, fko_errstr(res));
+            log_msg(LOG_ERR, "[%s] (stanza #%d) Unexpected error pulling SPA data from the context: %s",
+                spadat.pkt_source_ip, stanza_num, fko_errstr(res));
 
-            if(ctx != NULL)
-                fko_destroy(ctx);
             acc = acc->next;
             continue;
         }
@@ -564,13 +608,11 @@ incoming_spa(fko_srv_options_t *opts)
 
             ts_diff = abs(now_ts - spadat.timestamp);
 
-            if(ts_diff > atoi(opts->config[CONF_MAX_SPA_PACKET_AGE]))
+            if(ts_diff > conf_pkt_age)
             {
-                log_msg(LOG_WARNING, "(stanza #%d) SPA data time difference is too great (%i seconds).",
-                    stanza_num, ts_diff);
+                log_msg(LOG_WARNING, "[%s] (stanza #%d) SPA data time difference is too great (%i seconds).",
+                    spadat.pkt_source_ip, stanza_num, ts_diff);
 
-                if(ctx != NULL)
-                    fko_destroy(ctx);
                 acc = acc->next;
                 continue;
             }
@@ -583,11 +625,9 @@ incoming_spa(fko_srv_options_t *opts)
         spa_ip_demark = strchr(spadat.spa_message, ',');
         if(spa_ip_demark == NULL)
         {
-            log_msg(LOG_WARNING, "(stanza #%d) Error parsing SPA message string: %s",
-                stanza_num, fko_errstr(res));
+            log_msg(LOG_WARNING, "[%s] (stanza #%d) Error parsing SPA message string: %s",
+                spadat.pkt_source_ip, stanza_num, fko_errstr(res));
 
-            if(ctx != NULL)
-                fko_destroy(ctx);
             acc = acc->next;
             continue;
         }
@@ -598,16 +638,22 @@ incoming_spa(fko_srv_options_t *opts)
         if(strnlen(spadat.spa_message_src_ip,
                 MIN_IPV4_STR_LEN) < MIN_IPV4_STR_LEN)
         {
-            log_msg(LOG_WARNING, "(stanza #%d) Invalid source IP in SPA message, ignoring SPA packet",
-                stanza_num, fko_errstr(res));
+            log_msg(LOG_WARNING, "[%s] (stanza #%d) Invalid source IP in SPA message, ignoring SPA packet",
+                spadat.pkt_source_ip, stanza_num, fko_errstr(res));
 
             if(ctx != NULL)
-                fko_destroy(ctx);
-            acc = acc->next;
+            {
+                if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
+                    log_msg(LOG_WARNING,
+                        "[%s] (stanza #%d) fko_destroy() could not zero out sensitive data buffer.",
+                        spadat.pkt_source_ip, stanza_num, fko_errstr(res)
+                    );
+                ctx = NULL;
+            }
             break;
         }
 
-        strlcpy(spadat.spa_message_remain, spa_ip_demark+1, 1024);
+        strlcpy(spadat.spa_message_remain, spa_ip_demark+1, MAX_DECRYPTED_SPA_LEN);
 
         /* If use source IP was requested (embedded IP of 0.0.0.0), make sure it
          * is allowed.
@@ -617,12 +663,10 @@ incoming_spa(fko_srv_options_t *opts)
             if(acc->require_source_address)
             {
                 log_msg(LOG_WARNING,
-                    "(stanza #%d) Got 0.0.0.0 when valid source IP was required.",
-                    stanza_num
+                    "[%s] (stanza #%d) Got 0.0.0.0 when valid source IP was required.",
+                    spadat.pkt_source_ip, stanza_num
                 );
 
-                if(ctx != NULL)
-                    fko_destroy(ctx);
                 acc = acc->next;
                 continue;
             }
@@ -640,12 +684,10 @@ incoming_spa(fko_srv_options_t *opts)
             if(strcmp(spadat.username, acc->require_username) != 0)
             {
                 log_msg(LOG_WARNING,
-                    "(stanza #%d) Username in SPA data (%s) does not match required username: %s",
-                    stanza_num, spadat.username, acc->require_username
+                    "[%s] (stanza #%d) Username in SPA data (%s) does not match required username: %s",
+                    spadat.pkt_source_ip, stanza_num, spadat.username, acc->require_username
                 );
 
-                if(ctx != NULL)
-                    fko_destroy(ctx);
                 acc = acc->next;
                 continue;
             }
@@ -666,8 +708,6 @@ incoming_spa(fko_srv_options_t *opts)
                     stanza_num, spadat.pkt_source_ip
                 );
 
-                if(ctx != NULL)
-                    fko_destroy(ctx);
                 acc = acc->next;
                 continue;
             }
@@ -677,8 +717,6 @@ incoming_spa(fko_srv_options_t *opts)
                 stanza_num, spadat.pkt_source_ip
             );
 
-            if(ctx != NULL)
-                fko_destroy(ctx);
             acc = acc->next;
             continue;
 #endif
@@ -691,20 +729,18 @@ incoming_spa(fko_srv_options_t *opts)
             if(!acc->enable_cmd_exec)
             {
                 log_msg(LOG_WARNING,
-                    "(stanza #%d) SPA Command message are not allowed in the current configuration.",
-                    stanza_num
+                    "[%s] (stanza #%d) SPA Command message are not allowed in the current configuration.",
+                    spadat.pkt_source_ip, stanza_num
                 );
 
-                if(ctx != NULL)
-                    fko_destroy(ctx);
                 acc = acc->next;
                 continue;
             }
             else
             {
                 log_msg(LOG_INFO,
-                    "(stanza #%d) Processing SPA Command message: command='%s'.",
-                    stanza_num, spadat.spa_message_remain
+                    "[%s] (stanza #%d) Processing SPA Command message: command='%s'.",
+                    spadat.pkt_source_ip, stanza_num, spadat.spa_message_remain
                 );
 
                 /* Do we need to become another user? If so, we call
@@ -712,10 +748,8 @@ incoming_spa(fko_srv_options_t *opts)
                 */
                 if(acc->cmd_exec_user != NULL && strncasecmp(acc->cmd_exec_user, "root", 4) != 0)
                 {
-                    if(opts->verbose)
-                        log_msg(LOG_INFO, "(stanza #%d) Setting effective user to %s (UID=%i) before running command.",
-                            stanza_num, acc->cmd_exec_user, acc->cmd_exec_uid);
-
+                    log_msg(LOG_INFO, "[%s] (stanza #%d) Setting effective user to %s (UID=%i) before running command.",
+                        spadat.pkt_source_ip, stanza_num, acc->cmd_exec_user, acc->cmd_exec_uid);
 
                     res = run_extcmd_as(acc->cmd_exec_uid,
                                         spadat.spa_message_remain, NULL, 0, 0);
@@ -732,14 +766,21 @@ incoming_spa(fko_srv_options_t *opts)
 
                 if(opts->verbose > 1)
                     log_msg(LOG_WARNING,
-                        "(stanza #%d) CMD_EXEC: command returned %i",
-                        stanza_num, status);
+                        "[%s] (stanza #%d) CMD_EXEC: command returned %i",
+                        spadat.pkt_source_ip, stanza_num, status);
 
                 if(status != 0)
                     res = SPA_MSG_COMMAND_ERROR;
 
                 if(ctx != NULL)
-                    fko_destroy(ctx);
+                {
+                    if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
+                        log_msg(LOG_WARNING,
+                            "[%s] (stanza #%d) fko_destroy() could not zero out sensitive data buffer.",
+                            spadat.pkt_source_ip, stanza_num, fko_errstr(res)
+                        );
+                    ctx = NULL;
+                }
 
                 /* we processed the command on a matching access stanza, so we
                  * don't look for anything else to do with this SPA packet
@@ -757,12 +798,10 @@ incoming_spa(fko_srv_options_t *opts)
         if(! acc_check_port_access(acc, spadat.spa_message_remain))
         {
             log_msg(LOG_WARNING,
-                "(stanza #%d) One or more requested protocol/ports was denied per access.conf.",
-                stanza_num
+                "[%s] (stanza #%d) One or more requested protocol/ports was denied per access.conf.",
+                spadat.pkt_source_ip, stanza_num
             );
 
-            if(ctx != NULL)
-                fko_destroy(ctx);
             acc = acc->next;
             continue;
         }
@@ -773,13 +812,30 @@ incoming_spa(fko_srv_options_t *opts)
         */
         process_spa_request(opts, acc, &spadat);
         if(ctx != NULL)
-            fko_destroy(ctx);
+        {
+            if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
+                log_msg(LOG_WARNING,
+                    "[%s] (stanza #%d) fko_destroy() could not zero out sensitive data buffer.",
+                    spadat.pkt_source_ip, stanza_num
+                );
+            ctx = NULL;
+        }
         break;
     }
 
     if (raw_digest != NULL)
         free(raw_digest);
 
+    if(ctx != NULL)
+    {
+        if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
+            log_msg(LOG_WARNING,
+                "[%s] fko_destroy() could not zero out sensitive data buffer.",
+                spadat.pkt_source_ip
+            );
+        ctx = NULL;
+    }
+
     return;
 }
 
