@@ -7,9 +7,9 @@
  *
  * Purpose: Header file for utils.c fwknopd server program.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -60,11 +60,8 @@
 */
 void hex_dump(const unsigned char *data, const int size);
 char* dump_ctx(fko_ctx_t ctx);
-int is_base64(const unsigned char *buf, unsigned short int len);
+int is_base64(const unsigned char *buf, const unsigned short int len);
 int is_valid_dir(const char *path);
 int verify_file_perms_ownership(const char *file);
 
-size_t strlcat(char *dst, const char *src, size_t siz);
-size_t strlcpy(char *dst, const char *src, size_t siz);
-
 #endif  /* UTILS_H */
